import pkg from 'gulp'
const { src, dest, parallel, series, watch } = pkg

import browserSync   from 'browser-sync'
import gulpSass      from 'gulp-sass'
import * as dartSass from 'sass'
import postCss       from 'gulp-postcss'
import cssnano       from 'cssnano'
const  sassModule    = gulpSass(dartSass)
import concat        from 'gulp-concat'
import uglifyim      from 'gulp-uglify-es'
const  uglify        = uglifyim.default
import rename        from 'gulp-rename'
import {deleteAsync} from 'del'
import imageminfn    from 'gulp-imagemin'
import cache         from 'gulp-cache'
import autoprefixer  from 'autoprefixer'
import ftp           from 'vinyl-ftp'
import rsyncModule   from 'gulp-rsync'

function browsersync() {
	browserSync.init({
		server: {
			baseDir: 'app/',
		},
		ghostMode: { clicks: false },
		notify: false,
		online: true,
		// tunnel: 'yousutename', // Attempt to use the URL https://yousutename.loca.lt
	})
}

function js() {
	return src([
		'app/libs/jquery/dist/jquery.min.js',
		'app/js/common.js', // Всегда в конце
		])
	.pipe(concat('scripts.min.js'))
	.pipe(uglify()) // Минимизировать весь js (на выбор)
	.pipe(dest('app/js'))
	.pipe(browserSync.stream())
}

function sass() {
	return src('app/sass/**/*.sass')
	.pipe(sassModule())
	.pipe(postCss([
		autoprefixer({ grid: 'autoplace' }),
		cssnano({ preset: ['default', { discardComments: { removeAll: true } }] })
	]))
	.pipe(rename({ suffix: '.min', prefix : '' }))
	.pipe(dest('app/css'))
	.pipe(browserSync.stream())
}

function imagemin() {
	return src(['app/img/**/*'], { encoding: false })
		.pipe(imageminfn())
		.pipe(dest('dist/img/'))
}

async function removedist() { await deleteAsync('dist/**/*', { force: true }) }
async function clearcache() { cache.clearAll() }

function buildcopy() {
	return src([
		'app/*.html',
		'app/.htaccess',
		'{app/js,app/css}/*.min.*',
		'app/fonts/**/*'
	], { base: 'app/', encoding: false })
	.pipe(dest('dist'))
}

function deploy() {
	let conn = ftp.create({
		host:     'hostname.com',
		user:     'username',
		password: 'userpassword',
		parallel: 10
	});
	let globs = [
		'dist/**',
		// 'dist/.htaccess',
	]
	return src(globs, { buffer: false, encoding: false })
	.pipe(conn.dest('/path/to/folder/on/server'))
}

function rsync() {
	return src('dist/')
		.pipe(rsyncModule({
			root: 'dist/',
			hostname: 'username@yousite.com',
			destination: 'yousite/public_html/',
			clean: true, // Mirror copy with file deletion
			// include: ['*.htaccess'], // Includes files to deploy
			exclude: ['**/Thumbs.db', '**/*.DS_Store'], // Excludes files from deploy
			recursive: true,
			archive: true,
			silent: false,
			compress: true
		}))
}

function startwatch() {
	watch(['app/sass/**/*.sass'], { usePolling: true }, sass)
	watch(['libs/**/*.js', 'app/js/common.js'], { usePolling: true }, js)
	watch(['app/*.html'], { usePolling: true }).on('change', browserSync.reload)
}

export { js, sass, imagemin, deploy, rsync, removedist, clearcache }
export let build = series(removedist, imagemin, js, sass, buildcopy)

export default series(js, sass, parallel(browsersync, startwatch))
